@@ -21,7 +21,7 @@ use warnings FATAL => 'all';
 package [% class_v %];
 use base '[% root_class_v %]::DB::Base';
 
-__PACKAGE__->set_up_table('[% table_v %]', ColumnGroup => 'Essential');
+__PACKAGE__->set_up_table('[% table_v %]', { ColumnGroup => 'Essential' });
 
 1;
 ENDS
